@@ -200,6 +200,8 @@ class Prep(FilterModule):
                             help="sampling frequency in Hz")
         parser.add_argument("--goertzel", type=utilities.yesno,
                             help="use goertzel algorithm or fft")
+        parser.add_argument("--cache-size",type=int, default=60,
+                            help="pipe cache, should match sinefit rate")
         parser.description = ARGS_DESC
 
     n_output = 0
@@ -211,30 +213,43 @@ class Prep(FilterModule):
             if parsed_args.merge:
                 output = outputs['prep']
                 output.name = 'prep_watched'
-                output.enable_cache(60)  # 1 out per second
+                output.enable_cache(parsed_args.cache_size)  # default holds 1 second @ 60Hz
 
             else:
                 # expect prep-a, prep-b, prep-c output streams
                 ph_outputs = []
                 for ph in range(nphases):
                     output_pipe = outputs['prep-%c' % ('abc'[ph])]
-                    output_pipe.enable_cache(60)
+                    output_pipe.enable_cache(parsed_args.cache_size)
                     ph_outputs.append(output_pipe)
         except KeyError as e:
             raise helpers.FilterError("required inputs [iv, zero_crossings], outputs [prep] (or [prep-a,...])")
 
         while True:
+            # check for breaks in the input data
+            if zc.end_of_interval:
+                zc.consume_all()
+                if parsed_args.merge:
+                    await output.close_interval()
+                else:
+                    for p in ph_outputs:
+                        await p.close_interval()
+            # read new data
             try:
                 iv_data = await iv.read(flatten=True)
                 zc_data = await zc.read(flatten=True)
             except EmptyPipe:
                 break
 
+                await asyncio.sleep(0.2) # wait for more data
             # print("iv [%d]  %s->%s" % (len(iv_data), timestamp_to_human(iv_data[0, 0]),
             #                           timestamp_to_human(iv_data[-1, 0])))
             # print("zc [%d] %s->%s" % (len(zc_data), timestamp_to_human(zc_data[0, 0]),
             #                          timestamp_to_human(zc_data[-1, 0])))
             await asyncio.sleep(0.2)
+            # make sure both pipes have data
+            if len(zc_data) == 0 or len(iv_data)==0:
+                continue
             # set up default parameters:
             nharm = parsed_args.nharm
             nshift = parsed_args.nshift
@@ -247,10 +262,10 @@ class Prep(FilterModule):
             # check to make sure the the zc data overlaps with iv data
             # if we lose a voltage signal iv stalls looking for zc data
             # flush the old iv data because there are not any zc's for it
-
+           
             if min(zc_timestamps) > max(data_timestamps):
                 print("no zero crossings, dumping iv data")
-                iv.consume(len(iv_data))
+                iv.consume_all()
                 continue
             if rows < 2:
                 continue  # go back and get more data before we run
