@@ -92,6 +92,10 @@ class Sinefit(FilterModule):
                             help="reject frequencies above threshold")
         parser.add_argument("--min-amp", type="float", default=10.0,
                             help="reject signals below threshold")
+        parser.add_argument("--sfit4-iters", type="int", default=7,
+                            help="accuracy increases with higher values")
+        parser.add_argument("--cache-size", type="int", default=60,
+                            help="pipe cache, default holds 1 second @ 60Hz")
         parser.description = ARGS_DESC
 
     async def run(self, parsed_args, inputs, outputs, nrows=0):
@@ -110,18 +114,25 @@ class Sinefit(FilterModule):
             raise helpers.FilterError("missing stream %s" % e)
 
         # cache the sinefit pipe
-        zc.enable_cache(60) # 1 out per second
+        zc.enable_cache(parsed_args.cache_size) # default 1 second @ 60 Hz
         rows_processed = 0
         f_expected = parsed_args.frequency
         a_min = parsed_args.min_amp
         f_min = parsed_args.min_freq
         f_max = parsed_args.max_freq
+        sfit4_iters = parsed_args.sfit4_iters
         # 1 indexed but ts is col 0 so this is correct
         column = parsed_args.v_index
 
         while True:
             try:
-                data = await iv.read(flatten=True)
+                # check if previous read was the end of an interval
+                # if so discard any queued data and close the output interval
+                if iv.end_of_interval:
+                    iv.consume_all()
+                    await zc.close_interval()
+                
+                data = await iv.read(flatten=True)               
             except EmptyPipe:
                 break
             await asyncio.sleep(0.1)
@@ -157,7 +168,7 @@ class Sinefit(FilterModule):
                 t_min = timestamp_to_seconds(data[start, 0])
 
                 # Do 4-parameter sine wave fit
-                (A, f0, phi, C) = helpers.sfit4(this, fs)
+                (A, f0, phi, C) = helpers.sfit4(this, fs,num_iters = sfit4_iters)
                 # Check bounds.  If frequency is too crazy, ignore this window
                 if f0 < f_min or f0 > f_max:
                     warn.warn("frequency %s outside valid range %s - %s\n" %
@@ -219,6 +230,7 @@ class Sinefit(FilterModule):
             # Return the number of rows we've processed
             warn.reset(last_inserted_timestamp)
             iv.consume(start)
+           
             rows_processed += start
             if(nrows > 0 and rows_processed >= nrows):
                 break
